import Cookies from "js-cookie";
import { storageSession, storageLocal } from "@pureadmin/utils";
import { request } from "./http";

export interface DataInfo<T> {
  /** token */
  token: string;
  userinfo: {
    account: string;
    nickname: string;
    is_super: boolean;
    permission: string[];
  };
  // /** `accessToken`的过期时间（时间戳） */
  expireIn?: T;
  // /** 用于调用刷新accessToken的接口时所需的token */
  // refreshToken: string;
  // /** 用户名 */
  // username?: string;
  // /** 当前登陆用户的角色 */
  // roles?: Array<string>;
}

export const sessionKey = "u-mobile-info";
export const sessionMechanismTokenKey = "authorized-mechanism-mobile-token";
export const sessionSurveyTokenKey = "authorized-Survey-mobile-token";
export const TokenKey = "authorized-mobile-token";

export function getSurveyToken() {
  return storageLocal().getItem(sessionSurveyTokenKey);
}

export function setSurveyToken(tk) {
  storageLocal().setItem(sessionSurveyTokenKey, tk);
}
export function removeSurveyToken() {
  storageLocal().removeItem(sessionSurveyTokenKey);
}
export function getMechanismToken() {
  return storageLocal().getItem(sessionMechanismTokenKey);
}

export function setMechanismToken(tk) {
  storageLocal().setItem(sessionMechanismTokenKey, tk);
}
export function removeMechanismToken() {
  storageLocal().removeItem(sessionMechanismTokenKey);
}

/** 获取`token` */
export function getToken(): DataInfo<number> {
  // 此处与`TokenKey`相同，此写法解决初始化时`Cookies`中不存在`TokenKey`报错

  try {
    return Cookies.get(TokenKey)
      ? JSON.parse(Cookies.get(TokenKey))
      : storageLocal().getItem(sessionKey);
  } catch (error) {}
}

/**
 * @description 设置`token`以及一些必要信息并采用无感刷新`token`方案
 * 无感刷新：后端返回`accessToken`（访问接口使用的`token`）、`refreshToken`（用于调用刷新`accessToken`的接口时所需的`token`，`refreshToken`的过期时间（比如30天）应大于`accessToken`的过期时间（比如2小时））、`expires`（`accessToken`的过期时间）
 * 将`accessToken`、`expires`这两条信息放在key值为authorized-token的cookie里（过期自动销毁）
 * 将`username`、`roles`、`refreshToken`、`expires`这四条信息放在key值为`user-info`的sessionStorage里（浏览器关闭自动销毁）
 */
export async function setToken(data: DataInfo<number>) {
  return new Promise(async (resolve) => {
    let expires = 0;
    const { token } = data;
    console.log("token", token);

    // expires = new Date().getTime() + (data?.expireIn || 0) * 1000; // 如果后端直接设置时间戳，将此处代码改为expires = data.expires，然后把上面的DataInfo<Date>改成DataInfo<number>即可
    const cookieString = JSON.stringify({ token, expires });
    Cookies.set(TokenKey, cookieString);
    resolve(0);
  });
}

/** 删除`token`以及key值为`user-info`的session信息 */
export function removeToken() {
  Cookies.remove(TokenKey);
  // sessionStorage.clear();
}

/** 格式化token（jwt格式） */
export const formatToken = (token: string): string => {
  return token;
};
